from django.contrib.admin.util import NestedObjects
from django.contrib.auth.models import Group
from django.utils.text import capfirst
from django.utils.encoding import force_text

from guardian.shortcuts import remove_perm

from pbs.admin import get_permission_codename


def get_deleted_objects(objs, opts, user, admin_site, using):
    """
    Find all objects related to ``objs`` that should also be deleted. ``objs``
    must be a homogenous iterable of objects (e.g. a QuerySet).

    Returns a nested list of strings suitable for display in the
    template with the ``unordered_list`` filter.

    """
    collector = NestedObjects(using=using)
    collector.collect(objs)
    perms_needed = set()

    def format_callback(obj):
        opts = obj._meta

        return '%s: %s' % (capfirst(opts.verbose_name),
                           force_text(obj))

    to_delete = collector.nested(format_callback)

    protected = [format_callback(obj) for obj in collector.protected]

    return to_delete, perms_needed, protected


def update_permissions(prescription, admin_site, state, action=remove_perm):
    """
    Update a prescription's object permissions based on the state of the
    ePFP.
    """
    # Loop over each model admin and check when it should be disabled,
    # adding/removing permissions as necessary.
    from pbs.prescription.admin import SavePrescriptionMixin
    group = Group.objects.get(name="Users")
    for model, model_admin in admin_site._registry.items():
        if isinstance(model_admin, SavePrescriptionMixin):
            if model_admin.can_delete and model_admin.lock_after == state:
                opts = model_admin.opts
                perm = get_permission_codename("delete", opts)
                if hasattr(model_admin, "prescription_filter_field"):
                    options = {
                        model_admin.prescription_filter_field: prescription
                    }
                else:
                    options = {"prescription": prescription}
                for obj in model.objects.filter(**options):
                    action("%s.%s" % (opts.app_label, perm), group, obj)
